<?php
/* LogFormatter by Martin Austermeier
	gives your log files a standardized format.
	
	If you want to add new modules, search for "@@include logformatter classes" in this script.
	Please send your modules to <software@datendienst.com> too, so I can integrate them into the distribution version. Thanks.
*/

define("DEBUG", TRUE);

define("LF", "\n");
define("CRLF", "\r\n");
define("BR", "<br/>");
define("NEWLINE", CRLF .BR);
define("TAB", "\t");

function WriteLn($txt) {
	print($txt .CRLF);
}

function ExplodeQuoted($string, $separator=",") {
	$elements = explode($separator, $string);
	for ($i = 0; $i < count($elements); $i++) {
		$nquotes = substr_count($elements[$i], '"');
		if ($nquotes %2 == 1) {
			for ($j = $i+1; $j < count($elements); $j++) {
				if (substr_count($elements[$j], '"') %2 == 1) { // Look for an odd-number of quotes
					// Put the quoted string's pieces back together again
					array_splice($elements, $i, $j-$i+1,
							implode($separator, array_slice($elements, $i, $j-$i+1)));
					break;
				}
			}
		}
		if ($nquotes > 0) {
			// Remove first and last quotes, then merge pairs of quotes
			$qstr =& $elements[$i];
			$qstr = substr_replace($qstr, '', strpos($qstr, '"'), 1);
			$qstr = substr_replace($qstr, '', strrpos($qstr, '"'), 1);
			$qstr = str_replace('""', '"', $qstr);
		}
	}
	return $elements;
}


function String2Array ($string, $template){
// http://www.php.net/manual/de/function.split.php#57113
  #search defined dividers
  preg_match_all ("|%(.+)%|U", $template, $template_matches);
  #replace dividers with "real dividers"
  $template = preg_replace ("|%(.+)%|U", "(.+)", $template);
  #search matches
  preg_match ("|" . $template . "|", $string, $string_matches);
  #[template_match] => $string_match
  foreach ($template_matches[1] as $key => $value){
     $output[$value] = $string_matches[($key + 1)];
  }
  return $output;
}


function MonthNameToInt($monthName) {
// convert Jan, Feb, Mar etc -> 1,2,3
	$t=strtotime($monthName);	// "Sep" -> timestamp
	return(intval(date("m", $t)));
}


function DebugOut($name, $var, $doNewline=TRUE, $doHR=FALSE) {
	if (DEBUG) {
		echo $name ."=";
		if (isset($var)) {
			if (is_array($var)) {
				// print_nice($var);
				print_r($var);
			} else {
				echo "<" .$var .">";
				// print_nice($var);
			}
		} else {
			echo "(undefiniert)";
		}
		
		if ($doNewline) {
			echo CRLF;
		}
		if ($doHR) {
			echo "<hr>\n";
		}
	}
}


function Error($msg) {
	echo "** Fehler ** " .$msg .NEWLINE;
}

function LogError($msg) {
	echo "** Fehler ** " .$msg .NEWLINE;
	// @@@ spter in Log-Datei schreiben
}



/****************************************************************/

define("OUTPUT_NEWLINE", CRLF);	// output line separator
define("OUTPUT_SOFT_CR", "<SOFT_CR>");	// output multi-line message separator
define("FIELD_DELIMITER", "\t");	// TAB separated output fields
// define("FIELD_DELIMITER", ">\t<");	// TAB separated output fields

define("FORMAT_MESSAGES", 1);

/**************************** Register ******************************/

$handlers = array();

function Register($name, $handler) {
	global $handlers;
	$handlers[$name] = $handler;
}


/**************************** LogFormatter ******************************/
class LogFormatter {
	var $tag;
	var $header;
	var $startColSource;
	var $delimEndSource;
	var $lineBegin;
	var	$lineCount;
	
	function LogFormatter() {
		$this->tag = "";
		$this->header = "Date" .FIELD_DELIMITER ."Time" .FIELD_DELIMITER ."Tag" .FIELD_DELIMITER ."Source" .FIELD_DELIMITER ."Message";
		$this->startColSource = 16;
		$this->delimEndSource = ":";
		$this->lineCount = 0;
		$this->lineBegin = "";
	}

	
	function GetDateTime($line) {
	// Sep  1 15:30:10,
		$monthName = substr($line, 0, 3);
		$day = ltrim(substr($line, 4, 2));
		$time = ltrim(substr($line, 6, 9));
		$aTime = explode(":", $time);
		// DebugOut("monthName", $monthName); DebugOut("day", $day); DebugOut("time", $time);
		
		$month = MonthNameToInt($monthName);
		$year = date("Y"); // current year
		time();
		$timeStamp = mktime($aTime[0], $aTime[1], $aTime[2], $month, $day, $year);
		if ($timeStamp > time()) { // date after today
			$year--; // use previous year
			$timeStamp = mktime($aTime[0], $aTime[1], $aTime[2], $month, $day, $year);
		}
		$dateTime = date("Y-m-d" .FIELD_DELIMITER ."H:i:s", $timeStamp);
		
		return($dateTime);
	}
	
	function FormatLine($line) {
	// format:  mmm dd hh:mm:ss source: message 
	
		$ch = substr($line, 0, 1);
		if ($ch == $this->lineBegin) {	// [
			$line = substr($line, 1);	
		}
		
		$sRest = substr($line, $this->startColSource);

		$i = strpos($sRest, $this->delimEndSource);
		if ($i) {
			$sSource = substr($sRest, 0, $i);
			$sMsg = substr($sRest, $i+1);
		} else {	// no "source:" -> return $sRest
			$sSource = "";
			$sMsg = $sRest;
		}
		$sSource = trim($sSource);
		$sMsg = trim($sMsg);
		// DebugOut("sSource", $sSource);	DebugOut("sMsg", $sMsg);
		
		$result = $this->GetDateTime($line) .FIELD_DELIMITER .$this->tag .FIELD_DELIMITER .$sSource .FIELD_DELIMITER .$sMsg;
		return($result);
	}


	function OutputLoop($lines) {
		foreach($lines as $line) {
			$this->lineCount++;	
			
			$formatted = $this->FormatLine($line);
			
			if (!empty($formatted)) {
				echo $formatted .OUTPUT_NEWLINE;
			}
		}
	}
	
	
	function OutputFormatted($inFile) {
		echo $this->header .OUTPUT_NEWLINE;

		$this->OutputLoop(file($inFile));
	}
}


function Usage() {
global $handlers;

	fwrite(STDERR, $_SERVER["SCRIPT_FILENAME"] ." by Martin Austermeier" .OUTPUT_NEWLINE);
	fwrite(STDERR, "Usage: " .$_SERVER["SCRIPT_FILENAME"] ."[options] format <infile >outfile" .OUTPUT_NEWLINE);
	fwrite(STDERR, "options= " .OUTPUT_NEWLINE);
	fwrite(STDERR, "\t" ."--help" ."\t" ."this screen" .OUTPUT_NEWLINE);
	fwrite(STDERR, "format= " .OUTPUT_NEWLINE);
	foreach(array_keys($handlers) as $handler) {
		fwrite(STDERR, "\t" .$handler .OUTPUT_NEWLINE);
	}
	fwrite(STDERR, OUTPUT_NEWLINE);
	fwrite(STDERR, "Example: " .$_SERVER["SCRIPT_FILENAME"] ." messages </var/log/messages >/tmp/messages.txt" .OUTPUT_NEWLINE);
	
	fwrite(STDERR, OUTPUT_NEWLINE);
	exit(1); 
}


/**************** @@include logformatter classes ***********************/
include("logformatter-messages.inc");	// messages, syslog
include("logformatter-apache.inc");	// apache, apache-error
include("logformatter-vmware.inc");	//  vmware, vmware-serverd, vmware-hostd, vmware-vmauthd, vmware-proxy
include("logformatter-ken.inc");	// AVM KEN: ken, ken-mail, ken-proxy, ken-server, ken-map

/**************************** MAIN ******************************/

$inFile = "php://stdin";

$params = array(); 
$paramIndex = 0;
$options = array("help" => FALSE);

// command line: php SCRIPT_FILENAME FORMAT
for ($argIndex = 1; $argIndex < $argc; $argIndex++) {	// start with 1; 0 = scriptname
	$arg = $argv[$argIndex];
	if (substr($arg, 0, 1) == "-") { // store switch
		switch ($arg) {
			case "--help":
			case "-h":
			case "-?":
				$options["help"] = TRUE;
				break;
		}
	} else {
		$params[$paramIndex] = $arg;
		$paramIndex++;
	}
}


if (count($params) > 0) {
	$format = $params[0];	
} else {
	$format = "";
}

if ( ($options["help"]) 						// --help
|| (!isset($handlers[$format])) ) {	// unknown format
	Usage();
}

//   DebugOut("format", $format); DebugOut("handlers", $handlers);

$handler = $handlers[$format];
$formatter = new $handler;
$formatter->OutputFormatted($inFile);

?>